import { Type, constructorOf } from '@tsed/core';
import {
  GlobalProviders,
  InjectorService,
  ProviderScope,
  registerProvider
} from '@tsed/di';

import * as Electron from 'electron';

import { AppOptions } from './decorators/app-settings';
import { AppSettingsService } from './services/app-settings.service';
import { ElectronApp } from './decorators/electron-app';

export abstract class App {
  readonly injector: InjectorService;
  private startedAt = new Date();

  constructor(settings: Partial<AppOptions> = {}) {
    // create injector with initial configuration
    this.injector = this.createInjector(this.getConfiguration(this, settings));

    this.createElectronApp(this.injector);
  }

  get settings(): AppSettingsService {
    return this.injector.settings as AppSettingsService;
  }

  get electronApp(): ElectronApp {
    return this.injector.get<ElectronApp>(ElectronApp)!;
  }

  static async bootstrap(
    module: Type<App>,
    settings: Partial<AppOptions> = {}
  ): Promise<App> {
    const app = new module(settings);

    return app;
  }

  async start(): Promise<any> {
    try {
    } catch (err) {
      return Promise.reject(err);
    }
  }

  private createInjector(settings: Partial<AppOptions> = {}) {
    const injector = new InjectorService();
    injector.settings = this.createSettingsService(injector);
    // injector.logger = $log;

    // @ts-ignore
    injector.settings.set(settings);

    /* istanbul ignore next */
    if (injector.settings.env === 'test') {
      injector.logger.stop();
    }

    return injector;
  }

  private createSettingsService(injector: InjectorService): AppSettingsService {
    const provider = GlobalProviders.get(AppSettingsService)!.clone();

    provider.instance = injector.invoke<AppSettingsService>(provider.useClass);
    injector.addProvider(AppSettingsService, provider);

    return provider.instance as any;
  }

  private getConfiguration(module: any, configuration: any = {}) {
    const provider = GlobalProviders.get(constructorOf(module))!;

    return { ...provider.configuration, ...configuration };
  }

  private createElectronApp(injector: InjectorService): void {
    injector.forkProvider(ElectronApp);
  }
}

registerProvider({
  provide: ElectronApp,
  scope: ProviderScope.SINGLETON,
  global: true,
  useValue: Electron.app
});
